#include "MMC5983MA.h"
#include <MPython.h>

void MMC5983MA::begin()
{
    Wire.begin();
    mPython.begin();
    writeReg(MAG_ADDR, 0x0A, 0b00000000);
    writeReg(MAG_ADDR, 0x0B, 0b10110000);
    writeReg(MAG_ADDR, 0x09, 0b00100001);
}

void MMC5983MA::cal()
{
    display.fillScreen(0);
    display.setCursorLine(1);
    display.printLine("一、 水平放置掌控版");
    display.setCursorLine(2);
    display.printLine("      以垂直地面为轴");
    display.setCursorLine(3);
    display.printLine("      旋转几周");
    readMagRaw(rx, ry, rz);
    max_x = rx;
    min_x = max_x;
    max_y = ry;
    min_y = max_y;
    max_z = rz;
    min_z = max_z;
    unsigned long ticks_start = millis();
    while((millis()-ticks_start)<15000){
        readMagRaw(rx, ry, rz);
        min_x = min(rx, min_x);
        min_y = min(ry, min_y);
        max_x = max(rx, max_x);
        max_y = max(ry, max_y);
        delay(50);
    }
    xoffs = (min_x + max_x)/2;
    yoffs = (min_y + max_y)/2;
    display.fillScreen(0);
    display.setCursorLine(1);
    display.printLine("二、 竖直放置掌控版");
    display.setCursorLine(2);
    display.printLine("      以垂直地面为轴");
    display.setCursorLine(3);
    display.printLine("      旋转几周");
    readMagRaw(rx, ry, rz);
    ticks_start = millis();
    while((millis()-ticks_start)<15000){
        readMagRaw(rx, ry, rz);
        min_z = min(rz, min_z);
        max_z = max(rz, max_z);
        delay(50);
    }
    zoffs = (min_z + max_z)/2;
    display.fillScreen(0);
    display.setCursorLine(1);
    display.printLine("校准完成");
    delay(3000);
    display.fillScreen(0);
}

void MMC5983MA::peeling()
{
    readMagRaw(rx, ry, rz);
    peeling_x = rx;
    peeling_y = ry;
    peeling_z = rz;
    is_peeling = 1;
}

void MMC5983MA::clear_peeling()
{
    peeling_x = 0.0;
    peeling_y = 0.0;
    peeling_z = 0.0;
    is_peeling = 0;
}


float MMC5983MA::get_Xraw()
{
    readMagRaw(rx, ry, rz);
    return rx* 0.25;
}

float MMC5983MA::get_Yraw()
{
    readMagRaw(rx, ry, rz);
    return ry* 0.25;
}

float MMC5983MA::get_Zraw()
{
    readMagRaw(rx, ry, rz);
    return rz* 0.25;
}

float MMC5983MA::get_heading()
{
    float temp_x,temp_y,temp_z;
    readMagRaw(rx, ry, rz);
    temp_x = rx - xoffs;
    temp_y = ry - yoffs;
    temp_z = rz - zoffs;
    float heading = atan2(temp_y,-temp_x)*(180/3.141592)+180;
    return heading;
}

float MMC5983MA::get_field_strength()
{
    readMagRaw(rx, ry, rz);
    if(is_peeling == 1){
        return sqrt((rx - peeling_x)*(rx - peeling_x) + (ry - peeling_y)*(ry - peeling_y) + (rz - peeling_z)*(rz - peeling_z))*0.25;
    }
    return sqrt(rx * rx + ry *ry + rz *rz )*0.25;
}

void MMC5983MA::readMagRaw(int16_t &x, int16_t &y, int16_t &z){

  writeReg(MAG_ADDR, 0x09, 0b00001000);   //perform set pulse
  delay(5);
  writeReg(MAG_ADDR, 0x09, 0b00010000);   //perform reset pulse
  delay(5);
  writeReg(MAG_ADDR, 0x09, 0b00000001);   //request new ADC measurement
  delay(10);

  Wire.beginTransmission(MAG_ADDR);
  Wire.write(0x00);                       //start at X axis MSB
  Wire.endTransmission();
  Wire.requestFrom(MAG_ADDR, 6);          //read until Z axis LSB
  x = ((Wire.read() << 8) | Wire.read()) - 32768;   //16-bit unsigned format, shifted
  y = ((Wire.read() << 8) | Wire.read()) - 32768;   //16-bit unsigned format, shifted
  z = ((Wire.read() << 8) | Wire.read()) + 32768;   //Z axis seems to be inverted, not sure why
}

uint8_t MMC5983MA::readReg(uint8_t addr, uint8_t reg){

  Wire.beginTransmission(addr);
  Wire.write(reg);
  Wire.endTransmission();
  Wire.requestFrom(addr, 1);
  return Wire.read();
}

void MMC5983MA::writeReg(uint8_t addr, uint8_t reg, uint8_t val){

  Wire.beginTransmission(addr);
  Wire.write(reg);
  Wire.write(val);
  Wire.endTransmission();
}
